==> thx_core-0.2.10.tar.gz: thx_core-0.2.10/thx/core/management/commands/thx_dump_object.py <==
# -*- coding: utf-8 -*-

from __future__ import print_function

import logging
from logging import StreamHandler
import json

from django.core.exceptions import FieldError, ObjectDoesNotExist
from django.core.management.base import CommandError
from django.core.serializers import serialize

try:
    from django.db.models import loading  # python2?
except ImportError:
    from django.apps import apps as loading

from fixture_magic.compat import get_all_related_objects
from fixture_magic.utils import (add_to_serialize_list, serialize_me, seen,
                                 serialize_fully)

from fixture_magic.management.commands.dump_object import Command as FMCommand


class FMCommandMixin:
    """
    visit Thux formum and Django-fixture-magic for description:
    - https://forum.thux.dev/t/thx-dump-object/119
    - https://github.com/davedash/django-fixture-magic
    """

    def add_arguments(self, parser):
        super().add_arguments(parser)
        # -c enables logging using store_true
        parser.add_argument(
            '-c', '--console', action='store_true', default=False,
            help='Debug - write logging to console',
        )
        parser.add_argument(
            '-l', '--debug-level', default='info',
            help='Set debug level (debug, info, warnings) for console',
        )
        parser.add_argument(
            '-o', '--output-filepath', dest='output_filepath', default=False,
            help='Set in which file the dump output will be wrote',
        )
        # si potrebbe aggiugere anche il nome del logger per scegliere di loggare in un altro file

    output_transaction = True

    def check_options_input(self, **options):
        error_text = ('%s\nTry calling dump_object with --help argument or ' +
                      'use the following arguments:\n %s' % self.args)
        try:
            # verify input is valid
            try:
                app_label, model_name = options['model'].split('.')
            except AttributeError:
                raise CommandError("Specify model as `appname.modelname")
            query = options['query']
            ids = options['ids']
            if ids and query:
                raise CommandError(error_text % 'either use query or id list, not both')
            if not (ids or query):
                raise CommandError(error_text % 'must pass list of --ids or a json --query')
        except IndexError:
            raise CommandError(error_text % 'No object_class or filter clause supplied.')
        except ValueError:
            raise CommandError(
                error_text %
                "object_class must be provided in the following format: app_name.model_name"
            )
        except AssertionError:
            raise CommandError(error_text % 'No filter argument supplied.')

        kitchensink = options.get('kitchensink')
        follow_fk = options.get('follow_fk', True)
        serialize_format = options.get('format', 'json')

        natural_foreign = (options.get('natural', False) or
                           options.get('natural_foreign', False))
        natural_primary = (options.get('natural', False) or
                           options.get('natural_primary', False))

        output_file_path = options.get('output_filepath', False)

        return (
            app_label, model_name, query, ids, kitchensink, follow_fk, serialize_format,
            natural_foreign, natural_primary, output_file_path
        )

    @staticmethod
    def _get_objects(app_label, model_name, query, ids):
        try:
            Model = loading.get_model(app_label, model_name)
        except Exception as e:
            Model = loading.get_app_config(app_label).get_model(model_name)
        if query:
            objs = Model.objects.filter(**json.loads(query))
        else:
            if ids[0] == '*':
                objs = Model.objects.all()
            else:
                try:
                    parsers = int, long, str  # retrocompatibilità python 2
                except NameError:
                    parsers = int, str
                for parser in parsers:
                    try:
                        objs = Model.objects.filter(pk__in=map(parser, ids))
                    except ValueError:
                        pass
                    else:
                        break
                else:
                    objs = []
        return Model, objs

    @staticmethod
    def _kitchensink(Model, objs):
        fields = get_all_related_objects(Model)
        related_fields = [rel.get_accessor_name() for rel in fields]

        for obj in objs:
            for rel in related_fields:
                try:
                    if hasattr(getattr(obj, rel), 'all'):
                        add_to_serialize_list(getattr(obj, rel).all())
                    else:
                        add_to_serialize_list([getattr(obj, rel)])
                except FieldError:
                    pass
                except ObjectDoesNotExist:
                    pass
        return fields, related_fields

    @staticmethod
    def set_logger(options):
        logger = logging.getLogger('file')
        debug_level = options.get('debug_level')
        logger.setLevel(debug_level.upper())
        logger.handlers[0].setLevel(debug_level.upper())

        if options.get('console'):
            console_handler = StreamHandler()
            console_handler.setLevel(debug_level.upper())
            logger.addHandler(console_handler)
            logger.setLevel(debug_level.upper())

        return logger, debug_level


class Command(FMCommandMixin, FMCommand):
    """
    Visit Thux forum and Django-fixture-magic for description:
    - https://forum.thux.dev/t/thx-dump-object/119
    - https://github.com/davedash/django-fixture-magic
    """

    def handle(self, *args, **options):
        logger, debug_level = self.set_logger(options)
        try:
            getattr(logger, debug_level.lower())(f"Command thx_dump_object Start:\n {options}")
            # it's been choose to use check_options_input instead put all arguments into __init__ as self argument
            # to avoid duplicated argoment between option and self and because of develop requested limit.
            (
                app_label, model_name, query, ids, kitchensink, follow_fk, serialize_format,
                natural_foreign, natural_primary, output_file_path
            ) = self.check_options_input(**options)

            Model, objs = self._get_objects(app_label, model_name, query, ids)

            if kitchensink:
                fields, related_fields = self._kitchensink(Model, objs)

            add_to_serialize_list(objs)

            if follow_fk:
                serialize_fully()
            else:
                # reverse list to match output of serializez_fully
                serialize_me.reverse()

            if output_file_path:
                with open(output_file_path, 'w') as f:
                    f.write(
                        serialize(
                            serialize_format, [o for o in serialize_me if o is not None],
                            indent=4,
                            use_natural_foreign_keys=natural_foreign,
                            use_natural_primary_keys=natural_primary
                        )
                    )
                f.close()
            else:
                self.stdout.write(
                    serialize(
                        serialize_format, [o for o in serialize_me if o is not None],
                        indent=4,
                        use_natural_foreign_keys=natural_foreign,
                        use_natural_primary_keys=natural_primary
                    )
                )

            # Clear the list. Useful for when calling multiple
            # dump_object commands with a single execution of django
            del serialize_me[:]
            seen.clear()
            getattr(logger, debug_level.lower())(
                f"Command thx_dump_object: Finish, output type {output_file_path if output_file_path else 'console'}"
            )

        except CommandError as ce:
            logger.error(f"Command thx_dump_object CommandError: {ce}")
            self.stdout.write(self.style.ERROR(
                f'Execution failed: {ce}')
            )
        except Exception as e:
            logger.error(f"Command thx_dump_object Exception: {e}")
            self.stdout.write(self.style.ERROR(
                f'Execution failed: {e}')
            )

==> thx_core-0.2.14.tar.gz: thx_core-0.2.14/thx/core/management/commands/thx_dump_object.py <==
# -*- coding: utf-8 -*-

import json

from django.apps import apps
from django.core.exceptions import FieldError, ObjectDoesNotExist
from django.core.management.base import CommandError
from django.core.serializers import serialize
from fixture_magic.compat import get_all_related_objects
from fixture_magic.management.commands.dump_object import Command as FMCommand
from fixture_magic.utils import add_to_serialize_list, seen, serialize_fully, serialize_me

from thx.core.management.commands import ThxBaseCommand


class FMCommandMixin:
    """
    visit Thux formum and Django-fixture-magic for description:
    - https://forum.thux.dev/t/thx-dump-object/119
    - https://github.com/davedash/django-fixture-magic
    """

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '-o', '--output-filepath', dest='output_filepath', default=False,
            help='Set in which file the dump output will be wrote',
        )

    output_transaction = True

    def check_options_input(self, **options):
        error_text = ('%s\nTry calling dump_object with --help argument or ' +
                      'use the following arguments:\n %s' % self.args)
        try:
            # verify input is valid
            try:
                app_label, model_name = options['model'].split('.')
            except AttributeError:
                raise CommandError("Specify model as `appname.modelname")
            query = options['query']
            ids = options['ids']
            if ids and query:
                raise CommandError(error_text % 'either use query or id list, not both')
            if not (ids or query):
                raise CommandError(error_text % 'must pass list of --ids or a json --query')
        except IndexError:
            raise CommandError(error_text % 'No object_class or filter clause supplied.')
        except ValueError:
            raise CommandError(
                error_text %
                "object_class must be provided in the following format: app_name.model_name"
            )
        except AssertionError:
            raise CommandError(error_text % 'No filter argument supplied.')

        kitchensink = options.get('kitchensink')
        follow_fk = options.get('follow_fk', True)
        serialize_format = options.get('format', 'json')

        natural_foreign = (options.get('natural', False) or
                           options.get('natural_foreign', False))
        natural_primary = (options.get('natural', False) or
                           options.get('natural_primary', False))

        output_file_path = options.get('output_filepath', False)

        return (
            app_label, model_name, query, ids, kitchensink, follow_fk, serialize_format,
            natural_foreign, natural_primary, output_file_path
        )

    @staticmethod
    def _get_objects(app_label, model_name, query, ids):
        try:
            Model = apps.get_model(app_label, model_name)
        except Exception:
            Model = apps.get_app_config(app_label).get_model(model_name)
        if query:
            objs = Model.objects.filter(**json.loads(query))
        else:
            if ids[0] == '*':
                objs = Model.objects.all()
            else:
                parsers = int, str
                for parser in parsers:
                    try:
                        objs = Model.objects.filter(pk__in=map(parser, ids))
                    except ValueError:
                        pass
                    else:
                        break
                else:
                    objs = []
        return Model, objs

    @staticmethod
    def _kitchensink(model_class, objs):
        fields = get_all_related_objects(model_class)
        related_fields = [rel.get_accessor_name() for rel in fields]

        for obj in objs:
            for rel in related_fields:
                try:
                    if hasattr(getattr(obj, rel), 'all'):
                        add_to_serialize_list(getattr(obj, rel).all())
                    else:
                        add_to_serialize_list([getattr(obj, rel)])
                except FieldError:
                    pass
                except ObjectDoesNotExist:
                    pass
        return fields, related_fields


class Command(ThxBaseCommand, FMCommandMixin, FMCommand):
    """
    Visit Thux forum and Django-fixture-magic for description:
    - https://forum.thux.dev/t/thx-dump-object/119
    - https://github.com/davedash/django-fixture-magic
    """

    def handle(self, *args, **options):
        try:
            self.logger.info(f"Command thx_dump_object Start:\n {options}")
            # it's been chosen to use check_options_input instead put all arguments into __init__ as self argument
            # to avoid duplicated argument between option and self and because of develop requested limit.
            (
                app_label, model_name, query, ids, kitchensink, follow_fk, serialize_format,
                natural_foreign, natural_primary, output_file_path
            ) = self.check_options_input(**options)

            model_class, objs = self._get_objects(app_label, model_name, query, ids)

            if kitchensink:
                self._kitchensink(model_class, objs)

            add_to_serialize_list(objs)

            if follow_fk:
                serialize_fully()
            else:
                # reverse list to match output of serializez_fully
                serialize_me.reverse()

            if output_file_path:
                with open(output_file_path, 'w') as f:
                    f.write(
                        serialize(
                            serialize_format, [o for o in serialize_me if o is not None],
                            indent=4,
                            use_natural_foreign_keys=natural_foreign,
                            use_natural_primary_keys=natural_primary
                        )
                    )
            else:
                self.stdout.write(
                    serialize(
                        serialize_format, [o for o in serialize_me if o is not None],
                        indent=4,
                        use_natural_foreign_keys=natural_foreign,
                        use_natural_primary_keys=natural_primary
                    )
                )

            # Clear the list. Useful for when calling multiple
            # dump_object commands with a single execution of django
            del serialize_me[:]
            seen.clear()
            self.logger.info(
                f"Command thx_dump_object: Finish, output type {output_file_path if output_file_path else 'console'}"
            )

        except CommandError as ce:
            self.logger.error(f"Command thx_dump_object CommandError: {ce}")
            self.stdout.write(self.style.ERROR(f'Execution failed: {ce}'))
        except Exception as e:
            self.logger.error(f"Command thx_dump_object Exception: {e}")
            self.stdout.write(self.style.ERROR(f'Execution failed: {e}'))
